// Copyright Cryptape Technologies LLC.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

use std::fmt;

#[derive(Debug, PartialEq, Clone)]
pub enum AuthenticationError {
    NoTransactionPermission,
    NoContractPermission,
    NoCallPermission,
    InvalidTransaction,
}

impl fmt::Display for AuthenticationError {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        let printable = match *self {
            AuthenticationError::NoTransactionPermission => "No transaction permission.".to_owned(),
            AuthenticationError::NoContractPermission => "No create contract permision.".to_owned(),
            AuthenticationError::NoCallPermission => "No contract call permission.".to_owned(),
            AuthenticationError::InvalidTransaction => "Invalid transaction.".to_owned(),
        };
        write!(f, "{}", printable)
    }
}
